@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -38,25 +38,19 @@ public:
 	bool wordform_case;
 	bool print_word_forms;
 	bool print_only_first;
-	void setNullFlush(bool pNullFlush);
 
 	void testPR(std::ostream& output);
 
 protected:
-	bool nullFlush;
-	bool runningWithNullFlush;
-
 	void printReading(Reading* reading, std::ostream& output);
 	void printSingleWindow(SingleWindow* window, std::ostream& output);
 
-	void runGrammarOnTextWrapperNullFlush(std::istream& input, std::ostream& output);
-
 	void mergeMappings(Cohort& cohort);
 
 private:
 	void printReading(Reading* reading, std::ostream& output, ApertiumCasing casing, int firstlower);
-	void processReading(Reading* cReading, const UChar* reading_string);
-	void processReading(Reading* cReading, const UString& reading_string);
+	void processReading(Reading* cReading, UChar* reading_string, Tag* wform);
+	void processReading(Reading* cReading, UString& reading_string, Tag* wform);
 };
 }
 
