@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -57,11 +57,13 @@ public:
 	uint32_t is_pleft, is_pright;
 	SingleWindow* parent;
 	UString text;
+	UString wblank;
 	Cohort *prev, *next;
 	Reading* wread;
 	ReadingList readings;
 	ReadingList deleted;
 	ReadingList delayed;
+	ReadingList ignored;
 	typedef bc::flat_map<uint32_t, double> num_t;
 	num_t num_max, num_min;
 	uint32SortedVector dep_children;
@@ -92,6 +94,16 @@ public:
 	bool remRelation(uint32_t rel, uint32_t cohort);
 	void setRelated();
 
+	void unignoreAll() {
+		if (!ignored.empty()) {
+			for (auto& r : ignored) {
+				r->deleted = false;
+			}
+			readings.insert(readings.end(), ignored.begin(), ignored.end());
+			ignored.clear();
+		}
+	}
+
 private:
 	void updateMinMax();
 };
