@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -73,6 +73,8 @@ enum : uint64_t {
 	POS_RIGHTMOST      = (1ull << 35), // rr
 	POS_NO_BARRIER     = (1ull << 36), // N
 	POS_WITH           = (1ull << 37), // w
+	POS_LOOK_IGNORED   = (1ull << 38), // I
+	POS_OTHER          = (1ull << 39), // t
 
 	MASK_POS_DEP       = POS_DEP_PARENT | POS_DEP_SIBLING | POS_DEP_CHILD | POS_DEP_GLOB,
 	MASK_POS_DEPREL    = MASK_POS_DEP | POS_RELATION,
