@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -40,11 +40,6 @@ Grammar::Grammar()
   , lines(0)
   , verbosity_level(0)
   , total_time(0)
-  , rules_any(nullptr)
-  , sets_any(nullptr)
-  , delimiters(nullptr)
-  , soft_delimiters(nullptr)
-  , tag_any(0)
 {
 	// Nothing in the actual body...
 }
@@ -78,6 +73,9 @@ void Grammar::addSet(Set*& to) {
 	else if (!soft_delimiters && to->name == stringbits[S_SOFTDELIMITSET]) {
 		soft_delimiters = to;
 	}
+	else if (!text_delimiters && to->name == stringbits[S_TEXTDELIMITSET]) {
+		text_delimiters = to;
+	}
 	if (verbosity_level > 0 && to->name[0] == 'T' && to->name[1] == ':') {
 		u_fprintf(ux_stderr, "Warning: Set name %S looks like a misattempt of template usage on line %u.\n", to->name.c_str(), to->line);
 	}
@@ -408,7 +406,7 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 	return set->hash;
 }
 
-void Grammar::getTags(const Set& set, std::set<TagVector>& rv) {
+void Grammar::getTags(const Set& set, TagVectorSet& rv) const {
 	// ToDo: getTags() ought to account for other operators than OR
 	for (auto s : set.sets) {
 		getTags(*getSet(s), rv);
@@ -419,6 +417,28 @@ void Grammar::getTags(const Set& set, std::set<TagVector>& rv) {
 	trie_getTags(set.trie_special, rv, tv);
 }
 
+TagList Grammar::getTagList_Any(const Set& theSet) const {
+	TagList theTags;
+	getTagList_Any(theSet, theTags);
+	return theTags;
+}
+
+void Grammar::getTagList_Any(const Set& theSet, TagList& theTags) const {
+	if (theSet.type & (ST_SET_UNIFY | ST_TAG_UNIFY)) {
+		theTags.clear();
+		theTags.push_back(single_tags.find(tag_any)->second);
+	}
+	else if (!theSet.sets.empty()) {
+		for (auto iter : theSet.sets) {
+			getTagList_Any(*sets_list[iter], theTags);
+		}
+	}
+	else {
+		trie_getTagList(theSet.trie, theTags);
+		trie_getTagList(theSet.trie_special, theTags);
+	}
+}
+
 Rule* Grammar::allocateRule() {
 	return new Rule;
 }
@@ -644,10 +664,10 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	rules_any = nullptr;
 
 	for (auto iter : single_tags_list) {
-		if (iter->regexp && iter->tag[0] != '"' && iter->tag[0] != '<') {
+		if (iter->regexp && !is_textual(iter->tag)) {
 			regex_tags.insert(iter->regexp);
 		}
-		if ((iter->type & T_CASE_INSENSITIVE) && iter->tag[0] != '"' && iter->tag[0] != '<') {
+		if ((iter->type & T_CASE_INSENSITIVE) && !is_textual(iter->tag)) {
 			icase_tags.insert(iter);
 		}
 		if (is_binary) {
@@ -735,6 +755,9 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		if (soft_delimiters) {
 			soft_delimiters->markUsed(*this);
 		}
+		if (text_delimiters) {
+			text_delimiters->markUsed(*this);
+		}
 
 		contexts_t tosave;
 		for (auto& cntx : contexts) {
@@ -983,6 +1006,12 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 		//*/
 		bool needs = false;
+		if (r->sublist && sets_vstr.test(r->sublist->number)) {
+			needs = true;
+		}
+		if (r->maplist && sets_vstr.test(r->maplist->number)) {
+			needs = true;
+		}
 		if (r->dep_target && nk.count(r->dep_target)) {
 			needs = true;
 		}
