@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -84,12 +84,13 @@ public:
 	typedef std::unordered_map<uint32_t, boost::dynamic_bitset<>> sets_by_tag_t;
 	sets_by_tag_t sets_by_tag;
 
-	uint32IntervalVector* rules_any;
-	boost::dynamic_bitset<>* sets_any;
+	uint32IntervalVector* rules_any = nullptr;
+	boost::dynamic_bitset<>* sets_any = nullptr;
 
-	Set* delimiters;
-	Set* soft_delimiters;
-	uint32_t tag_any;
+	Set* delimiters = nullptr;
+	Set* soft_delimiters = nullptr;
+	Set* text_delimiters = nullptr;
+	uint32_t tag_any = 0;
 	uint32Vector preferred_targets;
 	uint32SortedVector reopen_mappings;
 	typedef bc::flat_map<uint32_t, uint32_t> parentheses_t;
@@ -116,7 +117,9 @@ public:
 	void addSetToList(Set* s);
 	void allocateDummySet();
 	uint32_t removeNumericTags(uint32_t s);
-	void getTags(const Set& set, std::set<TagVector>& rv);
+	void getTags(const Set& set, TagVectorSet& rv) const;
+	TagList getTagList_Any(const Set& theSet) const;
+	void getTagList_Any(const Set& theSet, TagList& theTags) const;
 
 	void addAnchor(const UChar* to, uint32_t at, bool primary = false);
 	void addAnchor(const UString& to, uint32_t at, bool primary = false);
