@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -57,6 +57,7 @@ GrammarApplicator::GrammarApplicator(std::ostream& ux_err)
   , split_mappings(false)
   , dep_has_spanned(false)
   , dep_delimit(0)
+  , dep_absolute(false)
   , dep_original(false)
   , dep_block_loops(true)
   , dep_block_crossing(false)
@@ -120,6 +121,10 @@ GrammarApplicator::~GrammarApplicator() {
 	}
 	grammar = nullptr;
 	ux_stderr = nullptr;
+
+	for (auto rx : text_delimiters) {
+		uregex_close(rx);
+	}
 }
 
 void GrammarApplicator::resetIndexes() {
@@ -149,9 +154,65 @@ void GrammarApplicator::setGrammar(Grammar* res) {
 	index_readingSet_yes.resize(grammar->sets_list.size());
 	index_readingSet_no.clear();
 	index_readingSet_no.resize(grammar->sets_list.size());
+
+	if (res->text_delimiters) {
+		TagList theTags;
+		trie_getTagList(res->text_delimiters->trie, theTags);
+		trie_getTagList(res->text_delimiters->trie_special, theTags);
+		for (auto& t : theTags) {
+			UParseError pe;
+			UErrorCode status = U_ZERO_ERROR;
+
+			uint32_t flags = (t->type & T_CASE_INSENSITIVE) ? UREGEX_CASE_INSENSITIVE : 0;
+			text_delimiters.push_back(uregex_open(t->tag.c_str(), static_cast<int32_t>(t->tag.size()), flags, &pe, &status));
+			if (status != U_ZERO_ERROR) {
+				u_fprintf(ux_stderr, "Error: uregex_open returned %s trying to parse pattern %S - cannot continue!\n", u_errorName(status), t->tag.c_str());
+				CG3Quit(1);
+			}
+		}
+	}
+}
+
+void GrammarApplicator::setTextDelimiter(UString rx) {
+	for (auto r : text_delimiters) {
+		uregex_close(r);
+	}
+	text_delimiters.clear();
+
+	if (rx.empty()) {
+		return;
+	}
+
+	bool icase = false;
+	if (rx.size() >= 3 && rx.front() == '/') {
+		rx.erase(rx.begin());
+		while (rx.back() == '/' || rx.back() == 'r' || rx.back() == 'i') {
+			if (rx.back() == 'i') {
+				icase = true;
+			}
+			else if (rx.back() == '/') {
+				rx.pop_back();
+				break;
+			}
+			rx.pop_back();
+		}
+	}
+
+	UParseError pe;
+	UErrorCode status = U_ZERO_ERROR;
+
+	uint32_t flags = icase ? UREGEX_CASE_INSENSITIVE : 0;
+	text_delimiters.push_back(uregex_open(rx.c_str(), static_cast<int32_t>(rx.size()), flags, &pe, &status));
+	if (status != U_ZERO_ERROR) {
+		u_fprintf(ux_stderr, "Error: uregex_open returned %s trying to parse pattern %S - cannot continue!\n", u_errorName(status), rx.c_str());
+		CG3Quit(1);
+	}
 }
 
 void GrammarApplicator::index() {
+	if (!add_spacing) {
+		ws[2] = '\n';
+	}
 	if (did_index) {
 		return;
 	}
@@ -159,6 +220,9 @@ void GrammarApplicator::index() {
 	if (grammar->ordered) {
 		ordered = true;
 	}
+	if (grammar->has_dep || dep_delimit) {
+		parse_dep = true;
+	}
 
 	if (!grammar->before_sections.empty()) {
 		uint32IntervalVector& m = runsections[-1];
@@ -288,7 +352,7 @@ Tag* GrammarApplicator::addTag(const UChar* txt, bool vstr) {
 	}
 
 	bool reflow = false;
-	if ((tag->type & T_REGEXP) && tag->tag[0] != '"' && tag->tag[0] != '<') {
+	if ((tag->type & T_REGEXP) && !is_textual(tag->tag)) {
 		if (grammar->regex_tags.insert(tag->regexp).second) {
 			for (auto titer : single_tags) {
 				if (titer.second->type & T_TEXTUAL) {
@@ -307,7 +371,7 @@ Tag* GrammarApplicator::addTag(const UChar* txt, bool vstr) {
 			}
 		}
 	}
-	if ((tag->type & T_CASE_INSENSITIVE) && tag->tag[0] != '"' && tag->tag[0] != '<') {
+	if ((tag->type & T_CASE_INSENSITIVE) && !is_textual(tag->tag)) {
 		if (grammar->icase_tags.insert(tag).second) {
 			for (auto titer : single_tags) {
 				if (titer.second->type & T_TEXTUAL) {
@@ -423,7 +487,10 @@ void GrammarApplicator::printReading(const Reading* reading, std::ostream& outpu
 		if (unicode_tags) {
 			pattern = local_utf_pattern;
 		}
-		if (!dep_has_spanned) {
+		if (dep_absolute) {
+			u_fprintf_u(output, pattern, reading->parent->global_number, pr->global_number);
+		}
+		else if (!dep_has_spanned) {
 			u_fprintf_u(output, pattern,
 			  reading->parent->local_number,
 			  pr->local_number);
@@ -455,7 +522,7 @@ void GrammarApplicator::printReading(const Reading* reading, std::ostream& outpu
 		if (!reading->parent->relations.empty()) {
 			for (auto miter : reading->parent->relations) {
 				for (auto siter : miter.second) {
-					u_fprintf(output, " R:%S:%u", grammar->single_tags.find(miter.first)->second->tag.c_str(), siter);
+					u_fprintf(output, " R:%S:%u", single_tags.find(miter.first)->second->tag.c_str(), siter);
 				}
 			}
 		}
@@ -477,8 +544,6 @@ void GrammarApplicator::printReading(const Reading* reading, std::ostream& outpu
 }
 
 void GrammarApplicator::printCohort(Cohort* cohort, std::ostream& output) {
-	constexpr UChar ws[] = { ' ', '\t', 0 };
-
 	if (cohort->local_number == 0) {
 		goto removed;
 	}
@@ -502,6 +567,8 @@ void GrammarApplicator::printCohort(Cohort* cohort, std::ostream& output) {
 	}
 	u_fputc('\n', output);
 
+	cohort->unignoreAll();
+
 	if (!split_mappings) {
 		mergeMappings(*cohort);
 	}
@@ -524,7 +591,7 @@ void GrammarApplicator::printCohort(Cohort* cohort, std::ostream& output) {
 removed:
 	if (!cohort->text.empty() && cohort->text.find_first_not_of(ws) != UString::npos) {
 		u_fprintf(output, "%S", cohort->text.c_str());
-		if (!ISNL(cohort->text[cohort->text.size() - 1])) {
+		if (!ISNL(cohort->text.back())) {
 			u_fputc('\n', output);
 		}
 	}
@@ -552,9 +619,9 @@ void GrammarApplicator::printSingleWindow(SingleWindow* window, std::ostream& ou
 		}
 	}
 
-	if (!window->text.empty()) {
+	if (!window->text.empty() && window->text.find_first_not_of(ws) != UString::npos) {
 		u_fprintf(output, "%S", window->text.c_str());
-		if (!ISNL(window->text[window->text.size() - 1])) {
+		if (!ISNL(window->text.back())) {
 			u_fputc('\n', output);
 		}
 	}
@@ -564,7 +631,17 @@ void GrammarApplicator::printSingleWindow(SingleWindow* window, std::ostream& ou
 		Cohort* cohort = window->cohorts[c];
 		printCohort(cohort, output);
 	}
-	u_fputc('\n', output);
+
+	if (!window->text_post.empty() && window->text_post.find_first_not_of(ws) != UString::npos) {
+		u_fprintf(output, "%S", window->text_post.c_str());
+		if (!ISNL(window->text_post.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
+	if (add_spacing) {
+		u_fputc('\n', output);
+	}
 	u_fflush(output);
 }
 
