@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -98,9 +98,11 @@ public:
 	bool is_conv;
 	bool split_mappings;
 	bool pipe_deleted = false;
+	bool add_spacing = true;
 
 	bool dep_has_spanned;
 	uint32_t dep_delimit;
+	bool dep_absolute;
 	bool dep_original;
 	bool dep_block_loops;
 	bool dep_block_crossing;
@@ -123,11 +125,13 @@ public:
 	void disableStatistics();
 
 	void setGrammar(Grammar* res);
+	void setTextDelimiter(UString rx);
 	void index();
 
 	virtual void runGrammarOnText(std::istream& input, std::ostream& output);
 
 	bool has_dep;
+	bool parse_dep = false;
 	uint32_t dep_highest_seen;
 	std::unique_ptr<Window> gWindow;
 	void reflowDependencyWindow(uint32_t max = 0);
@@ -177,6 +181,7 @@ protected:
 
 	UString span_pattern_latin;
 	UString span_pattern_utf;
+	UChar ws[4]{ ' ', '\t', 0, 0 };
 
 	uint32_t numLines;
 	uint32_t numWindows;
@@ -223,6 +228,7 @@ protected:
 
 	typedef bc::flat_map<uint32_t, Reading*> readings_plain_t;
 	readings_plain_t readings_plain;
+	std::vector<URegularExpression*> text_delimiters;
 
 	typedef bc::flat_map<uint32_t, const void*> unif_tags_t;
 	bc::flat_map<uint32_t, unif_tags_t*> unif_tags_rs;
