@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -867,7 +867,7 @@ inline bool GrammarApplicator::doesSetMatchCohort_helper(Cohort& cohort, Reading
 bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t set, dSMC_Context* context) {
 	bool retval = false;
 
-	if (!(!context || (context->options & (POS_LOOK_DELETED | POS_LOOK_DELAYED | POS_NOT))) && (set >= cohort.possible_sets.size() || !cohort.possible_sets.test(set))) {
+	if (!(!context || (context->options & (POS_LOOK_DELETED | POS_LOOK_DELAYED | POS_LOOK_IGNORED | POS_NOT))) && (set >= cohort.possible_sets.size() || !cohort.possible_sets.test(set))) {
 		return retval;
 	}
 
@@ -881,13 +881,16 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 		return retval;
 	}
 
-	ReadingList* lists[3] = { &cohort.readings };
+	ReadingList* lists[4] = { &cohort.readings };
 	if (context && (context->options & POS_LOOK_DELETED)) {
 		lists[1] = &cohort.deleted;
 	}
 	if (context && (context->options & POS_LOOK_DELAYED)) {
 		lists[2] = &cohort.delayed;
 	}
+	if (context && (context->options & POS_LOOK_IGNORED)) {
+		lists[3] = &cohort.ignored;
+	}
 
 	for (auto list : lists) {
 		if (list == nullptr) {
@@ -901,6 +904,9 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 					continue;
 				}
 			}
+			if (reading->active && context && (context->options & POS_OTHER)) {
+				continue;
+			}
 			if (doesSetMatchCohort_helper(cohort, *reading, *theset, context)) {
 				retval = true;
 			}
@@ -928,19 +934,22 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 bool GrammarApplicator::doesSetMatchCohortCareful(Cohort& cohort, const uint32_t set, dSMC_Context* context) {
 	bool retval = false;
 
-	if (!(!context || (context->options & (POS_LOOK_DELETED | POS_LOOK_DELAYED | POS_NOT))) && (set >= cohort.possible_sets.size() || !cohort.possible_sets.test(set))) {
+	if (!(!context || (context->options & (POS_LOOK_DELETED | POS_LOOK_DELAYED | POS_LOOK_IGNORED | POS_NOT))) && (set >= cohort.possible_sets.size() || !cohort.possible_sets.test(set))) {
 		return retval;
 	}
 
 	const Set* theset = grammar->sets_list[set];
 
-	ReadingList* lists[3] = { &cohort.readings };
+	ReadingList* lists[4] = { &cohort.readings };
 	if (context && (context->options & POS_LOOK_DELETED)) {
 		lists[1] = &cohort.deleted;
 	}
 	if (context && (context->options & POS_LOOK_DELAYED)) {
 		lists[2] = &cohort.delayed;
 	}
+	if (context && (context->options & POS_LOOK_IGNORED)) {
+		lists[3] = &cohort.ignored;
+	}
 
 	for (auto list : lists) {
 		if (list == nullptr) {
@@ -954,6 +963,9 @@ bool GrammarApplicator::doesSetMatchCohortCareful(Cohort& cohort, const uint32_t
 					continue;
 				}
 			}
+			if (reading->active && context && (context->options & POS_OTHER)) {
+				continue;
+			}
 			retval = doesSetMatchCohort_helper(cohort, *reading, *theset, context);
 			if (!retval) {
 				break;
