@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -38,13 +38,16 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 	if (test->pos & POS_ATTACH_TO) {
 		if (attach_to != cohort) {
 			// Clear readings for rules that care about readings
-			ReadingList* lists[3] = { &cohort->readings };
+			ReadingList* lists[4] = { &cohort->readings };
 			if (test->pos & POS_LOOK_DELETED) {
 				lists[1] = &cohort->deleted;
 			}
 			if (test->pos & POS_LOOK_DELAYED) {
 				lists[2] = &cohort->delayed;
 			}
+			if (test->pos & POS_LOOK_IGNORED) {
+				lists[3] = &cohort->ignored;
+			}
 
 			for (auto list : lists) {
 				if (list == nullptr) {
@@ -780,7 +783,12 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 	// Recursion may happen, so can't be static
 	CohortSet rels;
 
-	if (test->relation == grammar->tag_any) {
+	auto rtag = single_tags[test->relation];
+	while (rtag->type & T_VARSTRING) {
+		rtag = generateVarstringTag(rtag);
+	}
+
+	if (rtag->hash == grammar->tag_any) {
 		for (auto riter : current->relations) {
 			for (auto citer : riter.second) {
 				std::map<uint32_t, Cohort*>::iterator it = sWindow->parent->cohort_map.find(citer);
@@ -791,7 +799,7 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 		}
 	}
 	else {
-		RelationCtn::const_iterator riter = current->relations.find(test->relation);
+		RelationCtn::const_iterator riter = current->relations.find(rtag->hash);
 		if (riter != current->relations.end()) {
 			for (auto citer : riter->second) {
 				std::map<uint32_t, Cohort*>::iterator it = sWindow->parent->cohort_map.find(citer);
