@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -29,9 +29,35 @@
 
 namespace CG3 {
 
+inline bool testStringAgainst(const UString& str, std::vector<URegularExpression*>& rxs) {
+	bool rv = false;
+
+	for (size_t i = 0; i < rxs.size(); ++i) {
+		UErrorCode status = U_ZERO_ERROR;
+		uregex_setText(rxs[i], str.c_str(), static_cast<int32_t>(str.size()), &status);
+		if (status != U_ZERO_ERROR) {
+			CG3Quit(1);
+		}
+		status = U_ZERO_ERROR;
+		if (uregex_find(rxs[i], -1, &status)) {
+			rv = true;
+			if (i != 0) {
+				// Move regex that matched up front as it'll more likely match first next time
+				std::swap(rxs[0], rxs[i]);
+			}
+			break;
+		}
+		if (status != U_ZERO_ERROR) {
+			CG3Quit(1);
+		}
+	}
+
+	return rv;
+}
+
 void GrammarApplicator::initEmptySingleWindow(SingleWindow* cSWindow) {
 	Cohort* cCohort = alloc_cohort(cSWindow);
-	cCohort->global_number = 0;
+	cCohort->global_number = gWindow->cohort_counter++;
 	cCohort->wordform = tag_begin;
 
 	Reading* cReading = alloc_reading(cCohort);
@@ -227,7 +253,7 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 			}
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (!dep_delimit && grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at line %u - forcing break.\n", hard_limit, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -413,8 +439,6 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 			// Check whether the cohort still belongs to the window, as per --dep-delimit
 			if (!is_deleted && dep_delimit && dep_highest_seen && (cCohort->dep_self <= dep_highest_seen || cCohort->dep_self - dep_highest_seen > dep_delimit)) {
 				reflowDependencyWindow(cCohort->global_number);
-				gWindow->dep_map.clear();
-				gWindow->dep_window.clear();
 
 				for (auto iter : cSWindow->cohorts.back()->readings) {
 					addTagToReading(*iter, endtag);
@@ -433,6 +457,7 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 				lSWindow = cSWindow;
 				++numWindows;
 				did_soft_lookback = false;
+				dep_highest_seen = 0;
 
 				if (grammar->has_bag_of_tags) {
 					// This is slow and not 100% correct as it doesn't remove the tags from the previous window
@@ -625,7 +650,23 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 				}
 
 				if (line[0]) {
-					if (lCohort) {
+					if (lSWindow && lCohort && testStringAgainst(line, text_delimiters)) {
+						lSWindow->text_post += &line[0];
+
+						for (auto iter : cCohort->readings) {
+							addTagToReading(*iter, endtag);
+						}
+
+						splitAllMappings(all_mappings, *cCohort, true);
+						cSWindow->appendCohort(cCohort);
+						cCohort->line_number = numLines;
+						lSWindow = cSWindow;
+						cSWindow = nullptr;
+						cCohort = nullptr;
+						numCohorts++;
+						did_soft_lookback = false;
+					}
+					else if (lCohort) {
 						lCohort->text += &line[0];
 					}
 					else if (lSWindow) {
