@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -280,10 +280,10 @@ Reading* GrammarApplicator::get_sub_reading(Reading* tr, int sub_reading) {
 #define APPEND_TAGLIST_TO_READING(taglist, reading)                                  \
 	do {                                                                             \
 		for (auto tter : (taglist)) {                                                \
-			auto hash = tter->hash;                                                  \
 			while (tter->type & T_VARSTRING) {                                       \
 				tter = generateVarstringTag(tter);                                   \
 			}                                                                        \
+			auto hash = tter->hash;                                                  \
 			if (tter->type & T_MAPPING || tter->tag[0] == grammar->mapping_prefix) { \
 				mappings->push_back(tter);                                           \
 			}                                                                        \
@@ -297,30 +297,6 @@ Reading* GrammarApplicator::get_sub_reading(Reading* tr, int sub_reading) {
 		}                                                                            \
 	} while (0)
 
-#define INSERT_TAGLIST_TO_READING(iter, taglist, reading)                          \
-	do {                                                                           \
-		for (auto tag : (taglist)) {                                               \
-			if (tag->type & T_VARSTRING) {                                         \
-				tag = generateVarstringTag(tag);                                   \
-			}                                                                      \
-			if (tag->hash == grammar->tag_any) {                                   \
-				break;                                                             \
-			}                                                                      \
-			if (tag->type & T_MAPPING || tag->tag[0] == grammar->mapping_prefix) { \
-				mappings->push_back(tag);                                          \
-			}                                                                      \
-			else {                                                                 \
-				(iter) = (reading).tags_list.insert((iter), tag->hash);            \
-				++(iter);                                                          \
-			}                                                                      \
-			if (updateValidRules(rules, intersects, tag->hash, (reading))) {       \
-				iter_rules = intersects.find(rule.number);                         \
-				iter_rules_end = intersects.end();                                 \
-			}                                                                      \
-		}                                                                          \
-		reflowReading(reading);                                                    \
-	} while (0)
-
 /**
  * Applies the passed rules to the passed SingleWindow.
  *
@@ -449,6 +425,18 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 			if (cohort->readings.empty()) {
 				continue;
 			}
+			// If there's no reason to even attempt to restore, just skip it.
+			if (rule.type == K_RESTORE) {
+				if ((rule.flags & RF_DELAYED) && cohort->delayed.empty()) {
+					continue;
+				}
+				else if ((rule.flags & RF_IGNORED) && cohort->ignored.empty()) {
+					continue;
+				}
+				else if (!(rule.flags & (RF_DELAYED|RF_IGNORED)) && cohort->deleted.empty()) {
+					continue;
+				}
+			}
 			// If there is not even a remote chance the target set might match this cohort, skip it.
 			if (rule.sub_reading == 0 && (rule.target >= cohort->possible_sets.size() || !cohort->possible_sets.test(rule.target))) {
 				continue;
@@ -605,6 +593,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				target = nullptr;
 				mark = cohort;
 				uint8_t orz = regexgrps.first;
+				reading->active = true;
 				// Actually check if the reading is a valid target. First check if rule target matches...
 				if (rule.target && doesSetMatchReading(*reading, rule.target, (set.type & (ST_CHILD_UNIFY | ST_SPECIAL)) != 0)) {
 					bool regex_prop = true;
@@ -690,6 +679,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					++rule.num_fail;
 				}
 				readings_plain.insert(std::make_pair(reading->hash_plain, reading));
+				reading->active = false;
 
 				if (reading != cohort->readings[i]) {
 					cohort->readings[i]->matched_target = reading->matched_target;
@@ -735,6 +725,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 			const size_t state_num_readings = cohort->readings.size();
 			const size_t state_num_removed = cohort->deleted.size();
 			const size_t state_num_delayed = cohort->delayed.size();
+			const size_t state_num_ignored = cohort->ignored.size();
 			bool readings_changed = false;
 
 			auto add_cohort = [&](Cohort* cohort) {
@@ -900,7 +891,12 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					cohort->dep_children.erase(ch);
 				}
 				cohort->type |= CT_REMOVED;
-				cohort->prev->removed.push_back(cohort);
+				if (!cohort->prev->enclosed.empty()) {
+					cohort->prev->enclosed.back()->removed.push_back(cohort);
+				}
+				else {
+					cohort->prev->removed.push_back(cohort);
+				}
 				cohort->detach();
 				for (auto cm : current.parent->cohort_map) {
 					cm.second->dep_children.erase(cohort->dep_self);
@@ -913,6 +909,47 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				gWindow->rebuildCohortLinks();
 			};
 
+			auto make_relation_rtag = [&](Tag* tag, uint32_t id) {
+				UChar tmp[256] = { 0 };
+				u_sprintf(tmp, "R:%S:%u", tag->tag.c_str(), id);
+				auto nt = addTag(tmp);
+				return nt;
+			};
+
+			auto add_relation_rtag = [&](Cohort* cohort, Tag* tag, uint32_t id) {
+				auto nt = make_relation_rtag(tag, id);
+				for (auto& r : cohort->readings) {
+					addTagToReading(*r, nt);
+				}
+			};
+
+			auto set_relation_rtag = [&](Cohort* cohort, Tag* tag, uint32_t id) {
+				auto nt = make_relation_rtag(tag, id);
+				for (auto& r : cohort->readings) {
+					for (auto it = r->tags_list.begin(); it != r->tags_list.end();) {
+						auto utag = single_tags[*it]->tag;
+						if (utag[0] == 'R' && utag[1] == ':' && utag.size() > 2 + tag->tag.size() && utag[2 + tag->tag.size()] == ':' && utag.compare(2, tag->tag.size(), tag->tag) == 0) {
+							r->tags.erase(*it);
+							r->tags_textual.erase(*it);
+							r->tags_numerical.erase(*it);
+							r->tags_plain.erase(*it);
+							it = r->tags_list.erase(it);
+						}
+						else {
+							++it;
+						}
+					}
+					addTagToReading(*r, nt);
+				}
+			};
+
+			auto rem_relation_rtag = [&](Cohort* cohort, Tag* tag, uint32_t id) {
+				auto nt = make_relation_rtag(tag, id);
+				for (auto& r : cohort->readings) {
+					delTagFromReading(*r, nt);
+				}
+			};
+
 			// This loop acts on the result of the previous loop; letting the rules do their thing on the valid readings.
 			for (size_t i = 0; i < cohort->readings.size(); ++i) {
 				Reading* tr = get_sub_reading(cohort->readings[i], rule.sub_reading);
@@ -963,6 +1000,29 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					unif_sets = unif_sets_rs[reading.hash_plain];
 				}
 
+				auto insert_taglist_to_reading = [&](auto& iter, auto& taglist, auto& reading, auto& mappings) {
+					for (auto tag : taglist) {
+						if (tag->type & T_VARSTRING) {
+							tag = generateVarstringTag(tag);
+						}
+						if (tag->hash == grammar->tag_any) {
+							break;
+						}
+						if (tag->type & T_MAPPING || tag->tag[0] == grammar->mapping_prefix) {
+							mappings->push_back(tag);
+						}
+						else {
+							iter = reading.tags_list.insert(iter, tag->hash);
+							++iter;
+						}
+						if (updateValidRules(rules, intersects, tag->hash, reading)) {
+							iter_rules = intersects.find(rule.number);
+							iter_rules_end = intersects.end();
+						}
+					}
+					reflowReading(reading);
+				};
+
 				// Select is also special as it will remove non-matching readings
 				if (type == K_SELECT) {
 					if (good) {
@@ -1000,16 +1060,18 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					}
 					else if (type == K_JUMP) {
 						TRACE;
-						const Tag* to = getTagList(*rule.maplist).front();
-						uint32FlatHashMap::const_iterator it = grammar->anchors.find(to->hash);
+						auto to = getTagList(*rule.maplist).front();
+						while (to->type & T_VARSTRING) {
+							to = generateVarstringTag(to);
+						}
+						auto it = grammar->anchors.find(to->hash);
 						if (it == grammar->anchors.end()) {
 							u_fprintf(ux_stderr, "Warning: JUMP on line %u could not find anchor '%S'.\n", rule.line, to->tag.c_str());
 						}
 						else {
 							iter_rules = intersects.lower_bound(it->second);
-							--iter_rules;
+							goto repeat_rule;
 						}
-						goto repeat_rule;
 					}
 					else if (type == K_PROTECT) {
 						TRACE;
@@ -1021,7 +1083,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					}
 					else if (type == K_REMVARIABLE) {
 						TRACE;
-						const TagList names = getTagList(*rule.maplist);
+						auto names = getTagList(*rule.maplist);
 						for (auto tag : names) {
 							variables.erase(tag->hash);
 							if (rule.flags & RF_OUTPUT) {
@@ -1033,8 +1095,8 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					}
 					else if (type == K_SETVARIABLE) {
 						TRACE;
-						const TagList names = getTagList(*rule.maplist);
-						const TagList values = getTagList(*rule.sublist);
+						auto names = getTagList(*rule.maplist);
+						auto values = getTagList(*rule.sublist);
 						variables[names.front()->hash] = values.front()->hash;
 						if (rule.flags & RF_OUTPUT) {
 							current.variables_output.insert(names.front()->hash);
@@ -1377,6 +1439,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 						bool did_insert = false;
 						if (rule.childset1) {
+							bool found_spot = false;
 							auto spot_tags = ss_taglist.get();
 							getTagList(*grammar->sets_list[rule.childset1], spot_tags);
 							FILL_TAG_LIST(spot_tags);
@@ -1392,15 +1455,18 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 									++tmp;
 								}
 								if (found) {
+									found_spot = true;
 									break;
 								}
 							}
-							if (rule.flags & RF_AFTER) {
-								std::advance(it, spot_tags->size());
-							}
-							if (it != reading.tags_list.end()) {
-								INSERT_TAGLIST_TO_READING(it, *theTags, reading);
-								did_insert = true;
+							if (found_spot) {
+								if (rule.flags & RF_AFTER) {
+									std::advance(it, spot_tags->size());
+								}
+								if (it != reading.tags_list.end()) {
+									insert_taglist_to_reading(it, *theTags, reading, mappings);
+									did_insert = true;
+								}
 							}
 						}
 
@@ -1423,6 +1489,38 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 							readings_changed = true;
 						}
 					}
+					else if (rule.type == K_RESTORE) {
+						bool did_restore = false;
+						auto move_rs = [&](ReadingList& rl) {
+							for (size_t i = 0; i < rl.size();) {
+								if (doesSetMatchReading(*rl[i], rule.maplist->number)) {
+									rl[i]->deleted = false;
+									rl[i]->hit_by.push_back(rule.number);
+									cohort->readings.push_back(rl[i]);
+									rl.erase(rl.begin() + i);
+									did_restore = true;
+								}
+								else {
+									++i;
+								}
+							}
+						};
+
+						if (rule.flags & RF_DELAYED) {
+							move_rs(cohort->delayed);
+						}
+						else if (rule.flags & RF_IGNORED) {
+							move_rs(cohort->ignored);
+						}
+						else {
+							move_rs(cohort->deleted);
+						}
+
+						if (did_restore) {
+							TRACE;
+						}
+						break;
+					}
 					else if (rule.type == K_REPLACE) {
 						index_ruleCohort_no.clear();
 						TRACE;
@@ -1705,7 +1803,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								std::advance(it, spot_tags->size());
 							}
 							if (it != cReading->tags_list.end()) {
-								INSERT_TAGLIST_TO_READING(it, *theTags, *cReading);
+								insert_taglist_to_reading(it, *theTags, *cReading, mappings);
 								did_insert = true;
 							}
 						}
@@ -1723,6 +1821,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						index_ruleCohort_no.clear();
 
 						CohortSet withs;
+						target = cohort;
 						withs.insert(target);
 						Cohort* merge_at = target;
 						for (auto it : rule.dep_tests) {
@@ -2084,14 +2183,17 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 										attach->setRelated();
 										cohort->setRelated();
 										rel_did_anything |= cohort->addRelation(tter->hash, attach->global_number);
+										add_relation_rtag(cohort, tter, attach->global_number);
 									}
 									else if (type == K_SETRELATION) {
 										attach->setRelated();
 										cohort->setRelated();
 										rel_did_anything |= cohort->setRelation(tter->hash, attach->global_number);
+										set_relation_rtag(cohort, tter, attach->global_number);
 									}
 									else {
 										rel_did_anything |= cohort->remRelation(tter->hash, attach->global_number);
+										rem_relation_rtag(cohort, tter, attach->global_number);
 									}
 								}
 								if (rel_did_anything) {
@@ -2141,13 +2243,16 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 									if (type == K_ADDRELATIONS) {
 										cohort->setRelated();
 										rel_did_anything |= cohort->addRelation(tter->hash, attach->global_number);
+										add_relation_rtag(cohort, tter, attach->global_number);
 									}
 									else if (type == K_SETRELATIONS) {
 										cohort->setRelated();
 										rel_did_anything |= cohort->setRelation(tter->hash, attach->global_number);
+										set_relation_rtag(cohort, tter, attach->global_number);
 									}
 									else {
 										rel_did_anything |= cohort->remRelation(tter->hash, attach->global_number);
+										rem_relation_rtag(cohort, tter, attach->global_number);
 									}
 								}
 								for (auto tter : *sublist) {
@@ -2157,13 +2262,16 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 									if (type == K_ADDRELATIONS) {
 										attach->setRelated();
 										rel_did_anything |= attach->addRelation(tter->hash, cohort->global_number);
+										add_relation_rtag(attach, tter, cohort->global_number);
 									}
 									else if (type == K_SETRELATIONS) {
 										attach->setRelated();
 										rel_did_anything |= attach->setRelation(tter->hash, cohort->global_number);
+										set_relation_rtag(attach, tter, cohort->global_number);
 									}
 									else {
 										rel_did_anything |= attach->remRelation(tter->hash, cohort->global_number);
+										rem_relation_rtag(attach, tter, cohort->global_number);
 									}
 								}
 								if (rel_did_anything) {
@@ -2189,6 +2297,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				if (rule.flags & RF_DELAYED) {
 					cohort->delayed.insert(cohort->delayed.end(), removed.begin(), removed.end());
 				}
+				else if (rule.flags & RF_IGNORED) {
+					cohort->ignored.insert(cohort->ignored.end(), removed.begin(), removed.end());
+				}
 				else {
 					cohort->deleted.insert(cohort->deleted.end(), removed.begin(), removed.end());
 				}
@@ -2218,7 +2329,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 			}
 
 			// Cohort state has changed, so mark that the section did something
-			if (state_num_readings != cohort->readings.size() || state_num_removed != cohort->deleted.size() || state_num_delayed != cohort->delayed.size() || readings_changed) {
+			if (state_num_readings != cohort->readings.size() || state_num_removed != cohort->deleted.size() || state_num_delayed != cohort->delayed.size() || state_num_ignored != cohort->ignored.size() || readings_changed) {
 				if (!(rule.flags & RF_NOITERATE) && section_max_count != 1) {
 					section_did_something = true;
 				}
@@ -2351,8 +2462,6 @@ void GrammarApplicator::runGrammarOnWindow() {
 
 	if (has_dep) {
 		reflowDependencyWindow();
-		gWindow->dep_map.clear();
-		gWindow->dep_window.clear();
 		if (!input_eof && !gWindow->next.empty() && gWindow->next.back()->cohorts.size() > 1) {
 			for (auto cohort : gWindow->next.back()->cohorts) {
 				gWindow->dep_window[cohort->global_number] = cohort;
