@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -52,7 +52,7 @@ void GrammarWriter::printSet(std::ostream& output, const Set& curset) {
 		}
 		used_sets.insert(curset.number);
 		u_fprintf(output, "LIST %S = ", curset.name.c_str());
-		std::set<TagVector> tagsets[] = { trie_getTagsOrdered(curset.trie), trie_getTagsOrdered(curset.trie_special) };
+		TagVectorSet tagsets[] = { trie_getTagsOrdered(curset.trie), trie_getTagsOrdered(curset.trie_special) };
 		for (auto& tvs : tagsets) {
 			for (auto& tags : tvs) {
 				if (tags.size() > 1) {
@@ -63,7 +63,7 @@ void GrammarWriter::printSet(std::ostream& output, const Set& curset) {
 					u_fprintf(output, " ");
 				}
 				if (tags.size() > 1) {
-					u_fprintf(output, ")");
+					u_fprintf(output, ") ");
 				}
 			}
 		}
@@ -152,6 +152,9 @@ int GrammarWriter::writeGrammar(std::ostream& output) {
 			else if (s == grammar->soft_delimiters) {
 				s->name = stringbits[S_SOFTDELIMITSET];
 			}
+			else if (s == grammar->text_delimiters) {
+				s->name = stringbits[S_TEXTDELIMITSET];
+			}
 			else {
 				s->name.resize(12);
 				s->name.resize(u_sprintf(&s->name[0], "S%u", s->number));
@@ -415,6 +418,9 @@ void GrammarWriter::printContextualTest(std::ostream& to, const ContextualTest&
 		if (test.pos & POS_LOOK_DELAYED) {
 			u_fprintf(to, "d");
 		}
+		if (test.pos & POS_LOOK_IGNORED) {
+			u_fprintf(to, "I");
+		}
 		if (test.pos & POS_RELATION) {
 			u_fprintf(to, "r:%S", grammar->single_tags.find(test.relation)->second->tag.c_str());
 		}
