@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -191,7 +191,7 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 			} // end >= soft_limit
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at line %u - forcing break.\n", hard_limit, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -210,7 +210,7 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 
 				// Create 0th Cohort which serves as the beginning of sentence
 				cCohort = alloc_cohort(cSWindow);
-				cCohort->global_number = 0;
+				cCohort->global_number = gWindow->cohort_counter++;
 				cCohort->wordform = tag_begin;
 
 				cReading = alloc_reading(cCohort);
