@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -177,7 +177,7 @@ void NicelineApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 			}
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (!dep_delimit && grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at line %u - forcing break.\n", hard_limit, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -407,7 +407,10 @@ void NicelineApplicator::printReading(const Reading* reading, std::ostream& outp
 		if (unicode_tags) {
 			pattern = local_utf_pattern;
 		}
-		if (!dep_has_spanned) {
+		if (dep_absolute) {
+			u_fprintf_u(output, pattern, reading->parent->global_number, pr->global_number);
+		}
+		else if (!dep_has_spanned) {
 			u_fprintf_u(output, pattern,
 			  reading->parent->local_number,
 			  pr->local_number);
@@ -452,8 +455,6 @@ void NicelineApplicator::printReading(const Reading* reading, std::ostream& outp
 }
 
 void NicelineApplicator::printCohort(Cohort* cohort, std::ostream& output) {
-	constexpr UChar ws[] = { ' ', '\t', 0 };
-
 	if (cohort->local_number == 0) {
 		goto removed;
 	}
@@ -468,6 +469,8 @@ void NicelineApplicator::printCohort(Cohort* cohort, std::ostream& output) {
 		did_warn_statictags = true;
 	}
 
+	cohort->unignoreAll();
+
 	if (!split_mappings) {
 		mergeMappings(*cohort);
 	}
@@ -483,7 +486,7 @@ removed:
 	u_fputc('\n', output);
 	if (!cohort->text.empty() && cohort->text.find_first_not_of(ws) != UString::npos) {
 		u_fprintf(output, "%S", cohort->text.c_str());
-		if (!ISNL(cohort->text[cohort->text.size() - 1])) {
+		if (!ISNL(cohort->text.back())) {
 			u_fputc('\n', output);
 		}
 	}
@@ -492,7 +495,7 @@ removed:
 void NicelineApplicator::printSingleWindow(SingleWindow* window, std::ostream& output) {
 	if (!window->text.empty()) {
 		u_fprintf(output, "%S", window->text.c_str());
-		if (!ISNL(window->text[window->text.size() - 1])) {
+		if (!ISNL(window->text.back())) {
 			u_fputc('\n', output);
 		}
 	}
@@ -502,6 +505,14 @@ void NicelineApplicator::printSingleWindow(SingleWindow* window, std::ostream& o
 		Cohort* cohort = window->cohorts[c];
 		printCohort(cohort, output);
 	}
+
+	if (!window->text_post.empty()) {
+		u_fprintf(output, "%S", window->text_post.c_str());
+		if (!ISNL(window->text_post.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
 	u_fputc('\n', output);
 	u_fflush(output);
 }
