@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -42,17 +42,18 @@ public:
 	uint8_t matched_target : 1;
 	uint8_t matched_tests : 1;
 	uint8_t immutable : 1;
+	uint8_t active : 1;
 
-	uint32_t baseform;
-	uint32_t hash;
-	uint32_t hash_plain;
-	uint32_t number;
+	uint32_t baseform = 0;
+	uint32_t hash = 0;
+	uint32_t hash_plain = 0;
+	uint32_t number = 0;
 	uint32Bloomish tags_bloom;
 	uint32Bloomish tags_plain_bloom;
 	uint32Bloomish tags_textual_bloom;
-	Tag* mapping;
-	Cohort* parent;
-	Reading* next;
+	Tag* mapping = nullptr;
+	Cohort* parent = nullptr;
+	Reading* next = nullptr;
 	uint32Vector hit_by;
 	typedef uint32Vector tags_list_t;
 	tags_list_t tags_list;
@@ -71,6 +72,8 @@ public:
 	~Reading();
 	void clear();
 
+	Reading& operator=(const Reading& r);
+
 	Reading* allocateReading(Cohort* p);
 	Reading* allocateReading(const Reading& r);
 
