@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -49,8 +49,8 @@ private:
 
 	typedef std::vector<Tag*> TagVector;
 	uint32_t copyRelabelSetToGrammar(const Set* set);
-	TagVector transferTags(const TagVector tv_r);
-	void addTaglistsToSet(const std::set<TagVector> tvs, Set* set);
+	TagVector transferTags(const TagVector& tv_r);
+	void addTaglistsToSet(const TagVectorSet& tvs, Set* set);
 	void reindexSet(Set& s);
 	void addSetToGrammar(Set* s);
 	void relabelAsList(Set* set_g, const Set* set_r, const Tag* fromTag);
