@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -53,6 +53,8 @@ const UString g_flags[FLAGS_COUNT] = {
 	"REPEAT"_us,
 	"BEFORE"_us,
 	"AFTER"_us,
+	"IGNORED"_us,
+	"LOOKIGNORED"_us,
 };
 
 const UString keywords[KEYWORD_COUNT] = {
@@ -119,6 +121,7 @@ const UString keywords[KEYWORD_COUNT] = {
 	"PROTECT"_us,
 	"UNPROTECT"_us,
 	"MERGECOHORTS"_us,
+	"RESTORE"_us,
 };
 
 const UString stringbits[STRINGS_COUNT] = {
@@ -150,6 +153,8 @@ const UString stringbits[STRINGS_COUNT] = {
 	"IF"_us,
 	"_S_DELIMITERS_"_us,
 	"_S_SOFT_DELIMITERS_"_us,
+	"_S_TEXT_DELIMITERS_"_us,
+	"/(^|\\n)</s/r"_us,
 	">>>"_us,
 	"<<<"_us,
 	" LINK 0 "_us,
