@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -91,6 +91,7 @@ enum KEYWORDS : uint32_t {
 	K_PROTECT,
 	K_UNPROTECT,
 	K_MERGECOHORTS,
+	K_RESTORE,
 	KEYWORD_COUNT,
 };
 
@@ -124,6 +125,8 @@ enum : uint32_t {
 	S_IF,
 	S_DELIMITSET,
 	S_SOFTDELIMITSET,
+	S_TEXTDELIMITSET,
+	S_TEXTDELIM_DEFAULT,
 	S_BEGINTAG,
 	S_ENDTAG,
 	S_LINKZ,
@@ -216,6 +219,8 @@ enum : uint32_t {
 	FL_REPEAT,
 	FL_BEFORE,
 	FL_AFTER,
+	FL_IGNORED,
+	FL_LOOKIGNORED,
 	FLAGS_COUNT,
 };
 }
