@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -100,7 +100,7 @@ void Tag::parseTagRaw(const UChar* to, Grammar* grammar) {
 		if ((tmp[0] == '"' && tmp[length - 1] == '"') || (tmp[0] == '<' && tmp[length - 1] == '>')) {
 			type |= T_TEXTUAL;
 			if (tmp[0] == '"' && tmp[length - 1] == '"') {
-				if (tmp[1] == '<' && tmp[length - 2] == '>') {
+				if (tmp[1] == '<' && tmp[length - 2] == '>' && length > 4) {
 					type |= T_WORDFORM;
 				}
 				else {
@@ -116,7 +116,7 @@ void Tag::parseTagRaw(const UChar* to, Grammar* grammar) {
 		UErrorCode status = U_ZERO_ERROR;
 		uregex_setText(iter, tag.c_str(), static_cast<int32_t>(tag.size()), &status);
 		if (status == U_ZERO_ERROR) {
-			if (uregex_matches(iter, 0, &status)) {
+			if (uregex_find(iter, -1, &status)) {
 				type |= T_TEXTUAL;
 			}
 		}
@@ -146,7 +146,8 @@ void Tag::parseTagRaw(const UChar* to, Grammar* grammar) {
 	}
 	if (tag[0] == 'R' && tag[1] == ':') {
 		UChar relname[256];
-		if (u_sscanf(tag.c_str(), "R:%[^:]:%i", &relname, &dep_parent) == 2 && dep_parent != 0) {
+		dep_parent = std::numeric_limits<uint32_t>::max();
+		if (u_sscanf(tag.c_str(), "R:%[^:]:%i", &relname, &dep_parent) == 2 && dep_parent != std::numeric_limits<uint32_t>::max()) {
 			type |= T_RELATION;
 			Tag* reltag = grammar->allocateTag(relname);
 			comparison_hash = reltag->hash;
@@ -336,7 +337,7 @@ UString Tag::toUString(bool escape) const {
 		str += ':';
 	}
 
-	if (type & (T_CASE_INSENSITIVE | T_REGEXP) && tag[0] != '"') {
+	if (type & (T_CASE_INSENSITIVE | T_REGEXP) && !is_textual(tag)) {
 		str += '/';
 	}
 
@@ -352,7 +353,7 @@ UString Tag::toUString(bool escape) const {
 		str += tag;
 	}
 
-	if (type & (T_CASE_INSENSITIVE | T_REGEXP) && tag[0] != '"') {
+	if (type & (T_CASE_INSENSITIVE | T_REGEXP) && !is_textual(tag)) {
 		str += '/';
 	}
 	if (type & T_CASE_INSENSITIVE) {
