@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -100,6 +100,12 @@ void TextualParser::error(const char* str, const UChar* p) {
 	incErrorCount();
 }
 
+void TextualParser::error(const char* str, const UChar* p, const UString& msg) {
+	ux_bufcpy(nearbuf, p, 20);
+	u_fprintf(ux_stderr, str, filebase, result->lines, nearbuf, msg.c_str());
+	incErrorCount();
+}
+
 void TextualParser::error(const char* str, UChar c, const UChar* p) {
 	ux_bufcpy(nearbuf, p, 20);
 	u_fprintf(ux_stderr, str, filebase, c, result->lines, nearbuf);
@@ -126,6 +132,13 @@ void TextualParser::error(const char* str, const char* s, const UChar* S, const
 
 Tag* TextualParser::parseTag(const UChar* to, const UChar* p) {
 	Tag* tag = ::CG3::parseTag(to, p, *this);
+
+	if (tag->type & T_VARSTRING) {
+		if (tag->tag.find_first_of('{') == tag->tag.npos && tag->tag.find_first_of('$') == tag->tag.npos) {
+			error("%s: Error: Varstring tag %S had no variables, on line %u near `%S`!\n", tag->tag.c_str(), p);
+		}
+	}
+
 	if (!strict_tags.empty() && !strict_tags.count(tag->plain_hash)) {
 		if (tag->type & (T_ANY | T_VARSTRING | T_VSTR | T_META | T_VARIABLE | T_SET | T_PAR_LEFT | T_PAR_RIGHT | T_ENCL | T_TARGET | T_MARK | T_ATTACHTO | T_SAME_BASIC)) {
 			// Always allow...
@@ -153,7 +166,7 @@ Tag* TextualParser::parseTag(const UChar* to, const UChar* p) {
 				error("%s: Error: Baseform tag %S not on the strict-tags list, on line %u near `%S`!\n", tag->tag.c_str(), p);
 			}
 		}
-		else if (tag->tag[0] == '<' && tag->tag[tag->tag.size() - 1] == '>') {
+		else if (tag->tag.front() == '<' && tag->tag.back() == '>') {
 			if (strict_second) {
 				error("%s: Error: Secondary tag %S not on the strict-tags list, on line %u near `%S`!\n", tag->tag.c_str(), p);
 			}
@@ -175,7 +188,7 @@ Tag* TextualParser::addTag(Tag* tag) {
 
 void TextualParser::parseTagList(UChar*& p, Set* s) {
 	AST_OPEN(TagList);
-	std::set<TagVector> taglists;
+	TagVectorSet taglists;
 	bc::flat_map<Tag*, size_t> tag_freq;
 
 	while (*p && *p != ';' && *p != ')') {
@@ -234,8 +247,8 @@ void TextualParser::parseTagList(UChar*& p, Set* s) {
 			}
 
 			// sort + uniq the tags
-			std::sort(tags.begin(), tags.end());
-			tags.erase(std::unique(tags.begin(), tags.end()), tags.end());
+			std::sort(tags.begin(), tags.end(), compare_Tag());
+			tags.erase(std::unique(tags.begin(), tags.end(), equal_Tag()), tags.end());
 			// If this particular list of tags hasn't already been seen, then increment their frequency counts
 			if (taglists.insert(tags).second) {
 				for (auto t : tags) {
@@ -371,21 +384,21 @@ Set* TextualParser::parseSetInline(UChar*& p, Set* s) {
 				}
 
 				if (!set_ops.empty() && (set_ops.back() == S_SET_DIFF || set_ops.back() == S_SET_ISECT_U || set_ops.back() == S_SET_SYMDIFF_U)) {
-					std::set<TagVector> a;
+					TagVectorSet a;
 					result->getTags(*result->getSet(sets[sets.size() - 1]), a);
-					std::set<TagVector> b;
+					TagVectorSet b;
 					result->getTags(*result->getSet(sets[sets.size() - 2]), b);
 
 					std::vector<TagVector> r;
 					if (set_ops.back() == S_SET_ISECT_U) {
-						std::set_intersection(a.begin(), a.end(), b.begin(), b.end(), std::back_inserter(r));
+						std::set_intersection(a.begin(), a.end(), b.begin(), b.end(), std::back_inserter(r), compare_TagVector());
 					}
 					else if (set_ops.back() == S_SET_SYMDIFF_U) {
-						std::set_symmetric_difference(a.begin(), a.end(), b.begin(), b.end(), std::back_inserter(r));
+						std::set_symmetric_difference(a.begin(), a.end(), b.begin(), b.end(), std::back_inserter(r), compare_TagVector());
 					}
 					else if (set_ops.back() == S_SET_DIFF) {
 						// (b,a) because order matters for difference
-						std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::back_inserter(r));
+						std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::back_inserter(r), compare_TagVector());
 					}
 
 					set_ops.pop_back();
@@ -589,6 +602,10 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 			t.pos |= POS_LOOK_DELAYED;
 			++p;
 		}
+		if (*p == 'I') {
+			t.pos |= POS_LOOK_IGNORED;
+			++p;
+		}
 		if (*p == 'A') {
 			t.pos |= POS_ATTACH_TO;
 			++p;
@@ -605,6 +622,10 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 			t.pos |= POS_NUMERIC_BRANCH;
 			++p;
 		}
+		if (*p == 't') {
+			t.pos |= POS_OTHER;
+			++p;
+		}
 		if (*p == 'B') {
 			result->has_bag_of_tags = true;
 			t.pos |= POS_BAG_OF_TAGS;
@@ -629,7 +650,7 @@ void TextualParser::parseContextualTestPosition(UChar*& p, ContextualTest& t) {
 			auto c = static_cast<int32_t>(n - p);
 			u_strncpy(&gbuffers[0][0], p, c);
 			gbuffers[0][c] = 0;
-			Tag* tag = result->allocateTag(&gbuffers[0][0]);
+			Tag* tag = parseTag(&gbuffers[0][0], p);
 			t.relation = tag->hash;
 			p = n;
 		}
@@ -971,6 +992,9 @@ ContextualTest* TextualParser::parseContextualTestList(UChar*& p, Rule* rule, bo
 		if (rule->flags & RF_LOOKDELAYED) {
 			t->pos |= POS_LOOK_DELAYED;
 		}
+		if (rule->flags & RF_LOOKIGNORED) {
+			t->pos |= POS_LOOK_IGNORED;
+		}
 	}
 
 	t = result->addContextualTest(ot);
@@ -998,6 +1022,93 @@ void TextualParser::parseContextualDependencyTests(UChar*& p, Rule* rule) {
 	rule->addContextualTest(t, rule->dep_tests);
 }
 
+flags_t TextualParser::parseRuleFlags(UChar*& p) {
+	flags_t rv;
+
+	result->lines += SKIPWS(p);
+
+	auto lp = p;
+	bool setflag = true;
+	while (setflag) {
+		setflag = false;
+		for (uint32_t i = 0; i < FLAGS_COUNT; i++) {
+			UChar* op = p;
+			if (ux_simplecasecmp(p, g_flags[i])) {
+				p += g_flags[i].size();
+				rv.flags |= (1 << i);
+				setflag = true;
+
+				if (i == FL_SUB) {
+					if (*p != ':') {
+						goto undo_flag;
+					}
+					++p;
+					UChar* n = p;
+					result->lines += SKIPTOWS(n, 0, true);
+					auto c = static_cast<int32_t>(n - p);
+					u_strncpy(&gbuffers[0][0], p, c);
+					gbuffers[0][c] = 0;
+					p = n;
+					if (gbuffers[0][0] == '*') {
+						rv.sub_reading = GSR_ANY;
+					}
+					else {
+						u_sscanf(&gbuffers[0][0], "%d", &rv.sub_reading);
+					}
+				}
+
+				// Rule flags followed by letters or valid set characters should not be flags.
+				if (*p != '(' && *p != ';' && !ISSPACE(*p)) {
+				undo_flag:
+					rv.flags &= ~(1 << i);
+					p = op;
+					setflag = false;
+					break;
+				}
+
+				AST_OPEN(RuleFlag);
+				cur_ast->b = op;
+				AST_CLOSE(p);
+			}
+			result->lines += SKIPWS(p);
+			// If any of these is the next char, there cannot possibly be more rule options...
+			if (*p == '(' || *p == 'T' || *p == 't' || *p == ';') {
+				setflag = false;
+				break;
+			}
+		}
+	}
+
+	std::bitset<sizeof(rv.flags) * CHAR_BIT> bits;
+	for (auto excl : flag_excls) {
+		bits = rv.flags & excl;
+		if (bits.count() > 1) {
+			UString msg;
+			for (size_t i = 0; i < FLAGS_COUNT; ++i) {
+				if (bits.test(i)) {
+					msg += ' ';
+					msg += g_flags[i];
+				}
+			}
+			error("%s: Error: Line %u near `%S`: Mutually exclusive flags:%S!\n", lp, msg);
+		}
+	}
+
+	if (rv.flags & RF_UNMAPLAST && rv.flags & RF_SAFE) {
+		error("%s: Error: Line %u near `%S`: SAFE and UNMAPLAST are mutually exclusive!\n", lp);
+	}
+
+	if (rv.flags & RF_UNMAPLAST) {
+		rv.flags |= RF_UNSAFE;
+	}
+	if (rv.flags & RF_ENCL_FINAL) {
+		result->has_encl_final = true;
+	}
+	result->lines += SKIPWS(p);
+
+	return rv;
+}
+
 void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 	AST_OPEN(Rule);
 	Rule* rule = result->allocateRule();
@@ -1098,89 +1209,20 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 		AST_CLOSE(p);
 	}
 
-	lp = p;
-	bool setflag = true;
-	while (setflag) {
-		setflag = false;
-		for (uint32_t i = 0; i < FLAGS_COUNT; i++) {
-			UChar* op = p;
-			if (ux_simplecasecmp(p, g_flags[i])) {
-				p += g_flags[i].size();
-				rule->flags |= (1 << i);
-				setflag = true;
+	auto flags = parseRuleFlags(p);
+	rule->flags = flags.flags;
+	rule->sub_reading = flags.sub_reading;
 
-				if (i == FL_SUB) {
-					if (*p != ':') {
-						goto undo_flag;
-					}
-					++p;
-					UChar* n = p;
-					result->lines += SKIPTOWS(n, 0, true);
-					auto c = static_cast<int32_t>(n - p);
-					u_strncpy(&gbuffers[0][0], p, c);
-					gbuffers[0][c] = 0;
-					p = n;
-					if (gbuffers[0][0] == '*') {
-						rule->sub_reading = GSR_ANY;
-					}
-					else {
-						u_sscanf(&gbuffers[0][0], "%d", &rule->sub_reading);
-					}
-				}
-
-				// Rule flags followed by letters or valid set characters should not be flags.
-				if (*p != '(' && !ISSPACE(*p)) {
-				undo_flag:
-					rule->flags &= ~(1 << i);
-					p = op;
-					setflag = false;
-					break;
-				}
-
-				AST_OPEN(RuleFlag);
-				cur_ast->b = op;
-				AST_CLOSE(p);
-			}
-			result->lines += SKIPWS(p);
-			// If any of these is the next char, there cannot possibly be more rule options...
-			if (*p == '(' || *p == 'T' || *p == 't' || *p == ';') {
-				setflag = false;
-				break;
+	if (section_flags.flags) {
+		for (size_t i = 0; i < FLAGS_COUNT; ++i) {
+			auto f = (1ull << i);
+			if ((section_flags.flags & f) && !(rule->flags & _flags_excls[i])) {
+				rule->flags |= f;
 			}
 		}
 	}
-	if (rule->flags & MASK_ENCL) {
-		std::bitset<sizeof(rule->flags) * CHAR_BIT> bits(static_cast<uint64_t>(rule->flags & MASK_ENCL));
-		if (bits.count() > 1) {
-			error("%s: Error: Line %u near `%S`: ENCL_* are all mutually exclusive!\n", lp);
-		}
-	}
-	if (rule->flags & RF_KEEPORDER && rule->flags & RF_VARYORDER) {
-		error("%s: Error: Line %u near `%S`: KEEPORDER and VARYORDER are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_REMEMBERX && rule->flags & RF_RESETX) {
-		error("%s: Error: Line %u near `%S`: REMEMBERX and RESETX are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_NEAREST && rule->flags & RF_ALLOWLOOP) {
-		error("%s: Error: Line %u near `%S`: NEAREST and ALLOWLOOP are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_UNSAFE && rule->flags & RF_SAFE) {
-		error("%s: Error: Line %u near `%S`: SAFE and UNSAFE are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_UNMAPLAST && rule->flags & RF_SAFE) {
-		error("%s: Error: Line %u near `%S`: SAFE and UNMAPLAST are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_DELAYED && rule->flags & RF_IMMEDIATE) {
-		error("%s: Error: Line %u near `%S`: IMMEDIATE and DELAYED are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_WITHCHILD && rule->flags & RF_NOCHILD) {
-		error("%s: Error: Line %u near `%S`: WITHCHILD and NOCHILD are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_ITERATE && rule->flags & RF_NOITERATE) {
-		error("%s: Error: Line %u near `%S`: ITERATE and NOITERATE are mutually exclusive!\n", lp);
-	}
-	if (rule->flags & RF_BEFORE && rule->flags & RF_AFTER) {
-		error("%s: Error: Line %u near `%S`: BEFORE and AFTER are mutually exclusive!\n", lp);
+	if (section_flags.sub_reading && !rule->sub_reading) {
+		rule->sub_reading = section_flags.sub_reading;
 	}
 
 	if (!(rule->flags & (RF_ITERATE | RF_NOITERATE))) {
@@ -1191,13 +1233,6 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 	if (key == K_UNMAP && !(rule->flags & (RF_SAFE | RF_UNSAFE))) {
 		rule->flags |= RF_SAFE;
 	}
-	if (rule->flags & RF_UNMAPLAST) {
-		rule->flags |= RF_UNSAFE;
-	}
-	if (rule->flags & RF_ENCL_FINAL) {
-		result->has_encl_final = true;
-	}
-	result->lines += SKIPWS(p);
 
 	if (rule->flags & RF_WITHCHILD) {
 		AST_OPEN(RuleWithChildTarget);
@@ -1269,7 +1304,7 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 
 	result->lines += SKIPWS(p);
 	lp = p;
-	if (key == K_MAP || key == K_ADD || key == K_REPLACE || key == K_APPEND || key == K_SUBSTITUTE || key == K_COPY || key == K_ADDRELATIONS || key == K_ADDRELATION || key == K_SETRELATIONS || key == K_SETRELATION || key == K_REMRELATIONS || key == K_REMRELATION || key == K_SETVARIABLE || key == K_REMVARIABLE || key == K_ADDCOHORT || key == K_JUMP || key == K_SPLITCOHORT || key == K_MERGECOHORTS) {
+	if (key == K_MAP || key == K_ADD || key == K_REPLACE || key == K_APPEND || key == K_SUBSTITUTE || key == K_COPY || key == K_ADDRELATIONS || key == K_ADDRELATION || key == K_SETRELATIONS || key == K_SETRELATION || key == K_REMRELATIONS || key == K_REMRELATION || key == K_SETVARIABLE || key == K_REMVARIABLE || key == K_ADDCOHORT || key == K_JUMP || key == K_SPLITCOHORT || key == K_MERGECOHORTS || key == K_RESTORE) {
 		AST_OPEN(RuleMaplist);
 		swapper_false swp(no_isets, no_isets);
 		Set* s = parseSetInlineWrapper(p);
@@ -1522,18 +1557,28 @@ void TextualParser::parseRule(UChar*& p, KEYWORDS key) {
 	AST_CLOSE(p);
 }
 
-void TextualParser::parseAnchorish(UChar*& p) {
-	AST_OPEN(AnchorName);
-	UChar* n = p;
-	result->lines += SKIPTOWS(n, 0, true);
-	auto c = static_cast<int32_t>(n - p);
-	u_strncpy(&gbuffers[0][0], p, c);
-	gbuffers[0][c] = 0;
-	if (!only_sets) {
-		result->addAnchor(&gbuffers[0][0], static_cast<uint32_t>(result->rule_by_number.size()), true);
+void TextualParser::parseAnchorish(UChar*& p, bool rule_flags) {
+	// Initial : now forbidden to allow for section rule flags
+	if (*p != ':') {
+		AST_OPEN(AnchorName);
+		UChar* n = p;
+		result->lines += SKIPTOWS(n, 0, true);
+		auto c = static_cast<int32_t>(n - p);
+		u_strncpy(&gbuffers[0][0], p, c);
+		gbuffers[0][c] = 0;
+		if (!only_sets) {
+			result->addAnchor(&gbuffers[0][0], static_cast<uint32_t>(result->rule_by_number.size()), true);
+		}
+		p = n;
+		AST_CLOSE(p);
 	}
-	p = n;
-	AST_CLOSE(p);
+
+	result->lines += SKIPWS(p, ':');
+	if (rule_flags && *p == ':') {
+		++p;
+		section_flags = parseRuleFlags(p);
+	}
+
 	result->lines += SKIPWS(p, ';');
 	if (*p != ';') {
 		error("%s: Error: Expected closing ; on line %u near `%S` after anchor/section name!\n", p);
@@ -1609,6 +1654,42 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				}
 				AST_CLOSE(p + 1);
 			}
+			// TEXT-DELIMITERS
+			else if (IS_ICASE(p, "TEXT-DELIMITERS", "text-delimiters")) {
+				if (result->text_delimiters) {
+					error("%s: Error: Cannot redefine TEXT-DELIMITERS on line %u near `%S`!\n", p);
+				}
+				result->text_delimiters = result->allocateSet();
+				result->text_delimiters->line = result->lines;
+				result->text_delimiters->setName(stringbits[S_TEXTDELIMITSET]);
+				AST_OPEN(TextDelimiters);
+				p += 15;
+				result->lines += SKIPWS(p, '=');
+				if (*p != '=') {
+					error("%s: Error: Encountered a %C before the expected = on line %u near `%S`!\n", *p, p);
+				}
+				++p;
+				parseTagList(p, result->text_delimiters);
+				result->addSet(result->text_delimiters);
+				if (result->text_delimiters->trie.empty() && result->text_delimiters->trie_special.empty()) {
+					error("%s: Error: TEXT-DELIMITERS declared, but no definitions given, on line %u near `%S`!\n", p);
+				}
+
+				TagList theTags;
+				trie_getTagList(result->text_delimiters->trie, theTags);
+				trie_getTagList(result->text_delimiters->trie_special, theTags);
+				for (auto& tag : theTags) {
+					if (!(tag->type & T_REGEXP)) {
+						error("%s: Error: TEXT-DELIMITERS had non-regex tag on line %u near %S!\n", tag->tag.c_str());
+					}
+				}
+
+				result->lines += SKIPWS(p, ';');
+				if (*p != ';') {
+					error("%s: Error: Expected closing ; before line %u near `%S`!\n", p);
+				}
+				AST_CLOSE(p + 1);
+			}
 			// MAPPING-PREFIX
 			else if (IS_ICASE(p, "MAPPING-PREFIX", "mapping-prefix")) {
 				if (seen_mapping_prefix) {
@@ -1818,6 +1899,10 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 			else if (IS_ICASE(p, "MERGECOHORTS", "mergecohorts")) {
 				parseRule(p, K_MERGECOHORTS);
 			}
+			// RESTORE
+			else if (IS_ICASE(p, "RESTORE", "restore")) {
+				parseRule(p, K_RESTORE);
+			}
 			// SETS
 			else if (IS_ICASE(p, "SETS", "sets")) {
 				p += 4;
@@ -2230,7 +2315,7 @@ void TextualParser::parseFromUChar(UChar* input, const char* fname) {
 				AST_OPEN(Anchor);
 				p += 6;
 				result->lines += SKIPWS(p);
-				parseAnchorish(p);
+				parseAnchorish(p, false);
 				AST_CLOSE(p);
 			}
 			// INCLUDE
@@ -2724,6 +2809,21 @@ int TextualParser::parse_grammar(UString& data) {
 		}
 	}
 
+	for (auto rule : result->rule_by_number) {
+		if (rule->type == K_JUMP) {
+			auto to = result->getTagList_Any(*rule->maplist).front();
+			if (to->type & T_SPECIAL) {
+				// Relies on unification or varstrings, so can't check statically
+				continue;
+			}
+			auto it = result->anchors.find(to->hash);
+			if (it == result->anchors.end()) {
+				u_fprintf(ux_stderr, "Error: JUMP on line %u could not find anchor '%S'.\n", rule->line, to->tag.c_str());
+				++error_counter;
+			}
+		}
+	}
+
 	for (auto tag : result->single_tags_list) {
 		// ToDo: Remove for real ordered mode
 		if (tag->type & T_REGEXP_LINE) {
