@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -28,10 +28,6 @@ namespace CG3 {
 
 Window::Window(GrammarApplicator* p)
   : parent(p)
-  , cohort_counter(1)
-  , window_counter(0)
-  , window_span(0)
-  , current(nullptr)
 {
 }
 
