@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -42,12 +42,27 @@ int main(int argc, char* argv[]) {
 	}
 
 	U_MAIN_INIT_ARGS(argc, argv);
-	argc = u_parseArgs(argc, argv, NUM_OPTIONS, options);
+	argc = u_parseArgs(argc, argv, NUM_OPTIONS, options.data());
+
+	parse_opts("CG3_CONV_DEFAULT", options_default);
+	parse_opts("CG3_CONV_OVERRIDE", options_override);
+	for (size_t i = 0; i < options.size(); ++i) {
+		if (options_default[i].doesOccur && !options[i].doesOccur) {
+			options[i] = options_default[i];
+		}
+		if (options_override[i].doesOccur) {
+			options[i] = options_override[i];
+		}
+	}
 
 	if (argc < 0 || options[HELP1].doesOccur || options[HELP2].doesOccur) {
 		FILE* out = (argc < 0) ? stderr : stdout;
 		fprintf(out, "Usage: cg-conv [OPTIONS]\n");
 		fprintf(out, "\n");
+		fprintf(out, "Environment variable:\n");
+		fprintf(out, " CG3_CONV_DEFAULT: Sets default cmdline options, which the actual passed options will override.\n");
+		fprintf(out, " CG3_CONV_OVERRIDE: Sets forced cmdline options, which will override any passed option.\n");
+		fprintf(out, "\n");
 		fprintf(out, "Options:\n");
 
 		size_t longest = 0;
@@ -276,6 +291,9 @@ int main(int argc, char* argv[]) {
 	if (options[PIPE_DELETED].doesOccur) {
 		applicator.pipe_deleted = true;
 	}
+	if (options[NO_BREAK].doesOccur) {
+		applicator.add_spacing = false;
+	}
 
 	applicator.is_conv = true;
 	applicator.trace = true;
