@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -57,6 +57,12 @@ public:
 		{
 		}
 
+		const_iterator(const const_iterator& o)
+		  : fus(o.fus)
+		  , i(o.i)
+		{
+		}
+
 		const_iterator(const flat_unordered_map& fus, size_t i = 0)
 		  : fus(&fus)
 		  , i(i)
@@ -260,7 +266,7 @@ public:
 			return;
 		}
 
-		static thread_local container vals;
+		static container vals;
 		vals.resize(0);
 		vals.reserve(size_);
 		for (size_type i = 0, ie = capacity(); i < ie; ++i) {
