@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -39,9 +39,20 @@ int main(int argc, char* argv[]) {
 	srand(static_cast<uint32_t>(time(0)));
 
 	U_MAIN_INIT_ARGS(argc, argv);
-	argc = u_parseArgs(argc, argv, NUM_OPTIONS, options);
+	argc = u_parseArgs(argc, argv, NUM_OPTIONS, options.data());
 	FILE* out = stderr;
 
+	parse_opts("CG3_DEFAULT", options_default);
+	parse_opts("CG3_OVERRIDE", options_override);
+	for (size_t i = 0; i < options.size(); ++i) {
+		if (options_default[i].doesOccur && !options[i].doesOccur) {
+			options[i] = options_default[i];
+		}
+		if (options_override[i].doesOccur) {
+			options[i] = options_override[i];
+		}
+	}
+
 	if (options[VERSION_TOO_OLD].doesOccur) {
 		std::cout << CG3_TOO_OLD << std::endl;
 		return 0;
@@ -71,6 +82,10 @@ int main(int argc, char* argv[]) {
 	if (argc < 0 || options[HELP1].doesOccur || options[HELP2].doesOccur) {
 		fprintf(out, "Usage: vislcg3 [OPTIONS]\n");
 		fprintf(out, "\n");
+		fprintf(out, "Environment variable:\n");
+		fprintf(out, " CG3_DEFAULT: Sets default cmdline options, which the actual passed options will override.\n");
+		fprintf(out, " CG3_OVERRIDE: Sets forced cmdline options, which will override any passed option.\n");
+		fprintf(out, "\n");
 		fprintf(out, "Options:\n");
 
 		size_t longest = 0;
@@ -511,6 +526,17 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter* conv) {
 	if (options[HARD_LIMIT].doesOccur) {
 		applicator.hard_limit = abs(atoi(options[HARD_LIMIT].value));
 	}
+	if (options[TEXT_DELIMIT].doesOccur) {
+		CG3::UString rx = CG3::stringbits[CG3::S_TEXTDELIM_DEFAULT];
+		if (options[TEXT_DELIMIT].value) {
+			UErrorCode status = U_ZERO_ERROR;
+			size_t sn = strlen(options[TEXT_DELIMIT].value);
+			CG3::UString buf(sn * 3, 0);
+			auto len = ucnv_toUChars(conv, &buf[0], static_cast<int32_t>(sn * 3), options[TEXT_DELIMIT].value, static_cast<int32_t>(sn), &status);
+			rx.assign(buf.begin(), buf.begin() + len);
+		}
+		applicator.setTextDelimiter(rx);
+	}
 	if (options[DEP_DELIMIT].doesOccur) {
 		if (options[DEP_DELIMIT].value) {
 			applicator.dep_delimit = abs(atoi(options[DEP_DELIMIT].value));
@@ -518,6 +544,10 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter* conv) {
 		else {
 			applicator.dep_delimit = 10;
 		}
+		applicator.parse_dep = true;
+	}
+	if (options[DEP_ABSOLUTE].doesOccur) {
+		applicator.dep_absolute = true;
 	}
 	if (options[DEP_ORIGINAL].doesOccur) {
 		applicator.dep_original = true;
@@ -549,6 +579,9 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter* conv) {
 	if (options[STATISTICS].doesOccur) {
 		applicator.enableStatistics();
 	}
+	if (options[NO_BREAK].doesOccur) {
+		applicator.add_spacing = false;
+	}
 #ifndef HAVE_TICK_COUNTER
 	if (options[STATISTICS].doesOccur) {
 		std::cerr << "Error: Sorry, this build cannot gather statistics due to missing high resolution timers." << std::endl;
