@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -74,7 +74,9 @@ enum OPTIONS {
 	ALWAYS_SPAN,
 	SOFT_LIMIT,
 	HARD_LIMIT,
+	TEXT_DELIMIT,
 	DEP_DELIMIT,
+	DEP_ABSOLUTE,
 	DEP_ORIGINAL,
 	DEP_ALLOW_LOOPS,
 	DEP_BLOCK_CROSSING,
@@ -87,10 +89,11 @@ enum OPTIONS {
 	SHOW_TAG_HASHES,
 	SHOW_SET_HASHES,
 	DUMP_AST,
+	NO_BREAK,
 	NUM_OPTIONS,
 };
 
-UOption options[] = {
+std::array<UOption,NUM_OPTIONS> options{
 	UOPTION_DEF_D("help",                'h', UOPT_NO_ARG,       "shows this help"),
 	UOPTION_DEF_D("?",                   '?', UOPT_NO_ARG,       "shows this help"),
 	UOPTION_DEF_D("version",             'V', UOPT_NO_ARG,       "prints copyright and version information"),
@@ -145,7 +148,9 @@ UOption options[] = {
 	UOPTION_DEF_D("always-span",           0, UOPT_NO_ARG,       "forces scanning tests to always span across window boundaries"),
 	UOPTION_DEF_D("soft-limit",            0, UOPT_REQUIRES_ARG, "number of cohorts after which the SOFT-DELIMITERS kick in; defaults to 300"),
 	UOPTION_DEF_D("hard-limit",            0, UOPT_REQUIRES_ARG, "number of cohorts after which the window is forcefully cut; defaults to 500"),
+	UOPTION_DEF_D("text-delimit",        'T', UOPT_OPTIONAL_ARG, "additional delimit based on non-CG text, ensuring it isn't attached to a cohort; defaults to /(^|\\n)</s/r"),
 	UOPTION_DEF_D("dep-delimit",         'D', UOPT_OPTIONAL_ARG, "delimit windows based on dependency instead of DELIMITERS; defaults to 10"),
+	UOPTION_DEF_D("dep-absolute",          0, UOPT_NO_ARG,       "outputs absolute cohort numbers rather than relative ones"),
 	UOPTION_DEF_D("dep-original",          0, UOPT_NO_ARG,       "outputs the original input dependency tag even if it is no longer valid"),
 	UOPTION_DEF_D("dep-allow-loops",       0, UOPT_NO_ARG,       "allows the creation of circular dependencies"),
 	UOPTION_DEF_D("dep-no-crossing",       0, UOPT_NO_ARG,       "prevents the creation of dependencies that would result in crossing branches"),
@@ -159,7 +164,11 @@ UOption options[] = {
 	UOPTION_DEF_D("show-tag-hashes",       0, UOPT_NO_ARG,       "prints a list of tags and their hashes as they are parsed during the run"),
 	UOPTION_DEF_D("show-set-hashes",       0, UOPT_NO_ARG,       "prints a list of sets and their hashes; implies --grammar-only"),
 	UOPTION_DEF_D("dump-ast",              0, UOPT_NO_ARG,       "prints the grammar parse tree; implies --grammar-only"),
+	UOPTION_DEF_D("no-break",            'B', UOPT_NO_ARG,       "inhibits any extra whitespace in output"),
 };
+
+#include "options_parser.hpp"
+
 }
 
 #endif
