@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2018, GrammarSoft ApS
+* Copyright (C) 2007-2020, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -51,10 +51,11 @@ enum OPTIONS {
 	SUB_LTR,
 	ORDERED,
 	PIPE_DELETED,
+	NO_BREAK,
 	NUM_OPTIONS,
 };
 
-UOption options[] = {
+std::array<UOption, NUM_OPTIONS> options{
 	UOPTION_DEF_D("help",         'h', UOPT_NO_ARG,       "shows this help"),
 	UOPTION_DEF_D("?",            '?', UOPT_NO_ARG,       "shows this help"),
 	UOPTION_DEF_D("prefix",       'p', UOPT_REQUIRES_ARG, "sets the mapping prefix; defaults to @"),
@@ -64,14 +65,14 @@ UOption options[] = {
 	UOPTION_DEF_D("in-niceline",  'n', UOPT_NO_ARG,       "sets input format to Niceline CG"),
 	UOPTION_DEF_D("in-apertium",  'a', UOPT_NO_ARG,       "sets input format to Apertium"),
 	UOPTION_DEF_D("in-fst",       'f', UOPT_NO_ARG,       "sets input format to HFST/XFST"),
-	UOPTION_DEF_D("in-plain",     'p', UOPT_NO_ARG,       "sets input format to plain text"),
-	UOPTION_DEF_D("add-tags",       0, UOPT_NO_ARG,       "adds minimal analysis to readings (implies -p)"),
+	UOPTION_DEF_D("in-plain",     'x', UOPT_NO_ARG,       "sets input format to plain text"),
+	UOPTION_DEF_D("add-tags",       0, UOPT_NO_ARG,       "adds minimal analysis to readings (implies -x)"),
 	UOPTION_DEF_D("out-cg",       'C', UOPT_NO_ARG,       "sets output format to CG (default)"),
 	UOPTION_DEF_D("V",            'V', UOPT_NO_ARG,       nullptr),
 	UOPTION_DEF_D("out-apertium", 'A', UOPT_NO_ARG,       "sets output format to Apertium"),
 	UOPTION_DEF_D("out-matxin",   'M', UOPT_NO_ARG,       "sets output format to Matxin"),
 	UOPTION_DEF_D("out-niceline", 'N', UOPT_NO_ARG,       "sets output format to Niceline CG"),
-	UOPTION_DEF_D("out-plain",    'P', UOPT_NO_ARG,       "sets output format to plain text"),
+	UOPTION_DEF_D("out-plain",    'X', UOPT_NO_ARG,       "sets output format to plain text"),
 	UOPTION_DEF_D("wfactor",      'W', UOPT_REQUIRES_ARG, "FST weight factor (defaults to 1.0)"),
 	UOPTION_DEF_D("wtag",           0, UOPT_REQUIRES_ARG, "FST weight tag prefix (defaults to W)"),
 	UOPTION_DEF_D("sub-delim",    'S', UOPT_REQUIRES_ARG, "FST sub-reading delimiters (defaults to #)"),
@@ -79,7 +80,11 @@ UOption options[] = {
 	UOPTION_DEF_D("ltr",          'l', UOPT_NO_ARG,       "sets sub-reading direction to LTR"),
 	UOPTION_DEF_D("ordered",      'o', UOPT_NO_ARG,       "tag order matters mode"),
 	UOPTION_DEF_D("deleted",        0, UOPT_NO_ARG,       "read deleted readings as such, instead of as text"),
+	UOPTION_DEF_D("no-break",     'B', UOPT_NO_ARG,       "inhibits any extra whitespace in output"),
 };
+
+#include "options_parser.hpp"
+
 }
 
 #endif
